@@ -6,7 +6,7 @@ import os
 import time
 import sys
 
-from typing import Optional, TypeVar, Union, cast
+from typing import Optional, TypeVar, Union, cast, List, Dict
 
 from .constants import *
 from .control.constants import *
@@ -35,8 +35,8 @@ EV_IDENTIFY = "identify"
 EV_CONNECTED = "connected"
 EV_DISCONNECTED = "disconnected"
 
-_ACK_RETRIES = const(4)
-_ACK_DELAY = const(40)
+#_ACK_RETRIES = const(4)
+#_ACK_DELAY = const(40)
 
 RegType = TypeVar('RegType', bound=Union[PackType, PackTuple])
 
@@ -61,23 +61,33 @@ class Transport:
         # send a packet payload over the transport layer
         pass
 
+# TODO: replace randbytes
+
+
+def _rand_device_id():
+    r: List[int] = []
+    for i in range(8):
+        r.append(random.getrandbits(8))
+    return bytearray(r).hex()
+
 
 class Bus(EventEmitter):
     """A Jacdac bus that managed devices, service client, registers."""
 
     def __init__(self, transport: Transport, *, device_id: str = None) -> None:
         super().__init__(self)
-        self.devices: list['Device'] = []
-        self.unattached_clients: list['Client'] = []
-        self.all_clients: list['Client'] = []
-        self.servers: list['Server'] = []
+        self.devices: List['Device'] = []
+        self.unattached_clients: List['Client'] = []
+        self.all_clients: List['Client'] = []
+        self.servers: List['Server'] = []
+        self._event_counter = 0
         if device_id is None:
-            device_id = random.randbytes(8).hex()
+            device_id = _rand_device_id()
         self.self_device = Device(self, device_id, bytearray(4))
         self.process_thread = threading.Thread(target=self._process_task)
         self.transport = transport
         self._sendq: queue.Queue[bytes] = queue.Queue()
-        self.pending_tasks: list[asyncio.Task[None]] = []
+        self.pending_tasks: List[asyncio.Task[None]] = []
 
         self.loop = asyncio.new_event_loop()
 
@@ -116,7 +126,7 @@ class Bus(EventEmitter):
         # TODO: make this optional.
         brain = UniqueBrainServer(self)
 
-        def keep_task(t: asyncio.Task[None]):
+        def keep_task(t: 'asyncio.Task[None]'):
             if t.done():
                 t.result()  # throw exception if needed
                 return False
@@ -178,7 +188,7 @@ class Bus(EventEmitter):
         cutoff = now_ - 2000
         self.self_device.last_seen = now_  # make sure not to gc self
 
-        newdevs: list['Device'] = []
+        newdevs: List['Device'] = []
         for dev in self.devices:
             if dev.last_seen < cutoff:
                 dev._destroy()
@@ -218,7 +228,7 @@ class Bus(EventEmitter):
         dev.last_seen = now()
         log("reattaching services to {}; {}/{} to attach", dev,
             len(self.unattached_clients), len(self.all_clients))
-        new_clients: list['Client'] = []
+        new_clients: List['Client'] = []
         occupied = bytearray(dev.num_service_classes)
         for c in dev.clients:
             if c.broadcast:
@@ -330,16 +340,22 @@ class RawRegisterClient(EventEmitter):
     """A Jacdac register client
     """
 
-    def __init__(self, client: 'Client', code: int, pack_format: Union[str, None]) -> None:
+    def __init__(self, client: 'Client', code: int, pack_format: Optional[str]) -> None:
         super().__init__(client.bus)
         self.code = code
         self._data: Optional[bytearray] = None
         self._refreshed_at = 0
         self.client = client
         self.pack_format = pack_format
+        self.not_implemented = False
 
-    def current(self, refresh_ms: int = 500):
-        if self._refreshed_at + refresh_ms >= now():
+    def clear(self):
+        self._data = None
+        self._refreshed_at = 0
+        self.not_implemented = False
+
+    def current(self, refresh_ms: int = -1):
+        if refresh_ms < 0 or self._refreshed_at + refresh_ms >= now():
             return self._data
         return None
 
@@ -352,6 +368,9 @@ class RawRegisterClient(EventEmitter):
     def set_values(self, *args: PackType):
         if self.pack_format is None:
             raise RuntimeError("set_value not supported")
+        if not self.client.connected:
+            return
+
         data = jdpack(self.pack_format, *args)
 
         def send():
@@ -379,11 +398,13 @@ class RawRegisterClient(EventEmitter):
         return float(value) * scale if not value is None else None
 
     def _query(self):
+        if not self.client.connected:
+            return
         pkt = JDPacket(cmd=JD_GET(self.code))
         self.client.send_cmd(pkt)
 
     def refresh(self):
-        if self._refreshed_at < 0:
+        if not self.client.connected or self._refreshed_at < 0 or self.not_implemented:
             return  # already in progress
 
         def do_refresh():
@@ -414,6 +435,8 @@ class RawRegisterClient(EventEmitter):
 
     # can't be called from event handlers!
     def query(self, refresh_ms: int = 500):
+        if self.not_implemented:
+            return None
         curr = self.current(refresh_ms)
         if curr:
             return curr
@@ -425,17 +448,22 @@ class RawRegisterClient(EventEmitter):
         return self._data
 
     async def query_async(self, refresh_ms: int = 500):
+        if self.not_implemented:
+            return None
         curr = self.current(refresh_ms)
         if curr:
             return curr
         self.refresh()
+        # todo: test if changed
         await self.event(EV_CHANGE)
         if self._data is None:
             raise RuntimeError(
                 "Can't read reg #{} (from {})".format(hex(self.code), self.client))
         return self._data
 
-    def query_no_wait(self, refresh_ms: int = 500):
+    def query_no_wait(self, refresh_ms: int = -1):
+        if self.not_implemented:
+            return None
         curr = self.current(refresh_ms)
         if curr:
             return curr
@@ -443,6 +471,8 @@ class RawRegisterClient(EventEmitter):
         return self._data
 
     def handle_packet(self, pkt: JDPacket):
+        if self.not_implemented:
+            return
         if pkt.is_reg_get and pkt.reg_code == self.code:
             self._data = pkt.data
             self._refreshed_at = now()
@@ -620,7 +650,7 @@ class UniqueBrainServer(Server):
 class Client(EventEmitter):
     """Base class to define service clients."""
 
-    def __init__(self, bus: Bus, service_class: int, pack_formats: dict[int, str], role: str) -> None:
+    def __init__(self, bus: Bus, service_class: int, pack_formats: Dict[int, str], role: str) -> None:
         super().__init__(bus)
         self.broadcast = False
         self.service_class = service_class
@@ -629,7 +659,7 @@ class Client(EventEmitter):
         self.device: Optional['Device'] = None
         self.current_device:  Optional['Device'] = None
         self.role = role
-        self._registers: list[RawRegisterClient] = []
+        self._registers: List[RawRegisterClient] = []
         bus.unattached_clients.append(self)
         bus.all_clients.append(self)
 
@@ -644,15 +674,25 @@ class Client(EventEmitter):
                 return reg
         return None
 
+    @property
+    def connected(self) -> bool:
+        """Indicates if the client is a connected to a server"""
+        return True if self.device else False
+
     def register(self, code: int):
+        """Retreives the register by code"""
         r = self._lookup_register(code)
         if r is None:
-            pack_format = self.pack_formats[code]
-            # TODO: error policy?
+            pack_format = self._lookup_packformat(code)
             r = RawRegisterClient(self, code, pack_format)
             self._registers.append(r)
         return r
 
+    def _lookup_packformat(self, code: int) -> Optional[str]:
+        if code in self.pack_formats:
+            return self.pack_formats[code]
+        return None
+
     def handle_packet(self, pkt: JDPacket):
         pass
 
@@ -666,6 +706,7 @@ class Client(EventEmitter):
         self.handle_packet(pkt)
 
     def send_cmd(self, pkt: JDPacket):
+        """Sends a command packet to the server"""
         if self.current_device is None:
             return
         pkt.service_index = self.service_index
@@ -703,10 +744,34 @@ class Client(EventEmitter):
         if not self.broadcast:
             assert self.device
             self.device = None
+            for reg in self._registers:
+                reg.clear()
             self.bus.unattached_clients.append(self)
             self.bus.clear_attach_cache()
         self.emit(EV_DISCONNECTED)
 
+    def on_connect(self, handler: EventHandlerFn) -> UnsubscribeFn:
+        """Registers an event handler when the client connects to a server
+
+        Args:
+            handler (EventHandlerFn): function to run with client connects
+
+        Returns:
+            UnsubscribeFn: function to call to unregister handler
+        """
+        return self.on(EV_CONNECTED, handler)
+
+    def on_disconnect(self, handler: EventHandlerFn) -> UnsubscribeFn:
+        """Registers an event handler when the client disconnects from a server
+
+        Args:
+            handler (EventHandlerFn): function to run with client connects
+
+        Returns:
+            UnsubscribeFn: function to call to unregister handler
+        """
+        return self.on(EV_DISCONNECTED, handler)
+
     def on_event(self, code: int, handler: EventHandlerFn) -> UnsubscribeFn:
         """Registers an event handler for the given event code
 
@@ -732,7 +797,78 @@ class Client(EventEmitter):
         return self.on(EV_EVENT, cb)
 
 
-_JD_CONTROL_ANNOUNCE_FLAGS_RESTART_COUNTER_STEADY = const(0xf)
+class SensorClient(Client):
+    """A client for a sensor service"""
+
+    def __init__(self, bus: Bus, service_class: int, pack_formats: Dict[int, str], role: str, *, preferred_interval: int = None) -> None:
+        super().__init__(bus, service_class, pack_formats, role)
+        self.preferred_interval = preferred_interval
+
+    @property
+    def streaming_samples(self) -> Optional[int]:
+        """Queries the current estimated streaming samples value"""
+        return self.register(JD_REG_STREAMING_SAMPLES).value()
+
+    @property
+    def streaming_interval(self) -> Optional[int]:
+        return self.register(JD_REG_STREAMING_INTERVAL).value()
+
+    @property
+    def streaming_preferred_interval(self) -> Optional[int]:
+        return self.register(JD_REG_STREAMING_PREFERRED_INTERVAL).value()
+
+    def refresh_reading(self) -> None:
+        if self._should_refresh_streaming_samples():
+            self.register(JD_REG_STREAMING_SAMPLES).set_values(0xff)
+
+    def _lookup_packformat(self, code: int) -> Optional[str]:
+        if code == JD_REG_STREAMING_SAMPLES:
+            return "u8"
+        elif code == JD_REG_STREAMING_INTERVAL:
+            return "u32"
+        elif code == JD_REG_STREAMING_PREFERRED_INTERVAL:
+            return "u32"
+        return Client._lookup_packformat(self, code)
+
+    def _should_refresh_streaming_samples(self) -> bool:
+        readingReg = self.register(JD_REG_READING)
+        if readingReg._refreshed_at < 0:
+            # refresh in progress
+            return False
+
+        samplesReg = self.register(JD_REG_STREAMING_SAMPLES)
+        if samplesReg._refreshed_at < 0:
+            # refresh in progress
+            return False
+
+        samples = samplesReg.value()
+        # check if we have some value
+        MIN_SAMPLES = 16
+        if samples is None or samples < MIN_SAMPLES:
+            return True
+
+        interval = self._reading_interval()
+
+        # haven't seen samples in a while
+        if now() > readingReg._refreshed_at + 3 * interval:
+            return True
+
+        # check if the streaming is consumed
+        if now() > samplesReg._refreshed_at + max(0, samples - MIN_SAMPLES) * interval:
+            return True
+        return False
+
+    def _reading_interval(self) -> int:
+        """Resolves the best refresh interval for streaming"""
+        interval = self.streaming_interval
+        if interval:
+            return interval
+        interval = self.streaming_preferred_interval
+        if interval:
+            return interval
+        if self.preferred_interval:
+            return self.preferred_interval
+        return JD_STREAMING_DEFAULT_INTERVAL
 
 
 class Device(EventEmitter):
@@ -742,7 +878,7 @@ class Device(EventEmitter):
         super().__init__(bus)
         self.device_id = device_id
         self.services = services
-        self.clients: list[Client] = []
+        self.clients: List[Client] = []
         self.last_seen = now()
         self._event_counter: Optional[int] = None
         self._ctrl_client: Optional[Client] = None
@@ -762,7 +898,7 @@ class Device(EventEmitter):
 
     @property
     def reset_count(self):
-        return self.announce_flags & _JD_CONTROL_ANNOUNCE_FLAGS_RESTART_COUNTER_STEADY
+        return self.announce_flags & JD_CONTROL_ANNOUNCE_FLAGS_RESTART_COUNTER_STEADY
 
     @property
     def packet_count(self):
@@ -814,6 +950,17 @@ class Device(EventEmitter):
         self.last_seen = now()
         self.emit(EV_PACKET_RECEIVE, pkt)
 
+        if pkt.service_command == JD_CMD_COMMAND_NOT_IMPLEMENTED:
+            cmd = util.u16(pkt.data, 0)
+            if cmd >> 12 == CMD_GET_REG >> 12:
+                reg_code = cmd & CMD_REG_MASK
+                srv_index = pkt.service_index
+                for c in self.clients:
+                    if c.service_index == srv_index:
+                        c.register(reg_code).not_implemented = True
+                        break
+            return
+
         service_class = self.service_class_at(pkt.service_index)
         if not service_class or service_class == 0xffffffff:
             return
