@@ -14,6 +14,15 @@ JD_FRAME_FLAG_COMMAND = const(0x01)
 JD_FRAME_FLAG_ACK_REQUESTED = const(0x02)
 JD_FRAME_FLAG_IDENTIFIER_IS_SERVICE_CLASS = const(0x04)
 
+JD_REGISTER_POLL_STREAMING_INTERVAL = 5000
+JD_REGISTER_POLL_FIRST_REPORT_INTERVAL = 400
+JD_REGISTER_POLL_REPORT_INTERVAL = 5001
+JD_REGISTER_POLL_REPORT_MAX_INTERVAL = 60000
+JD_REGISTER_POLL_REPORT_VOLATILE_INTERVAL = 1000
+JD_REGISTER_POLL_REPORT_VOLATILE_MAX_INTERVAL = 5000
+JD_REGISTER_OPTIONAL_POLL_COUNT = 3
+JD_STREAMING_DEFAULT_INTERVAL = 50
+
 # Registers 0x001-0x07f - r/w common to all services
 # Registers 0x080-0x0ff - r/w defined per-service
 # Registers 0x100-0x17f - r/o common to all services
